# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Inquire undocumented functions"""


class inq_function:
    def ndinqr(self, node, key, pname="__tmpvar__", **kwargs):
        """Get information about a node.

        **Secondary Functions:**
        Set current node pointer to this node.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        node : int
            node number.
            It should be 0 for key=11, ``DB_NUMDEFINED``,
            ``DB_NUMSELECTED``, ``DB_MAXDEFINED``, and
            ``DB_MAXRECLENG``.

        key : int
            key as to information needed about the node.

            * DB_SELECTED - return select status:

              * 0 - node is undefined.
              * -1 - node is unselected.
              * 1 - node is selected.

            * ``DB_NUMDEFINED`` - return number of defined nodes
            * ``DB_NUMSELECTED`` - return number of selected nodes
            * ``DB_MAXDEFINED`` - return highest node number defined
            * ``DB_MAXRECLENG`` - return maximum record length (dp words)
            * 2, return length (dp words)
            * 3,
            * 4, pointer to first data word
            * 11, return void percent (integer)
            * 17, pointer to start of index
            * 117, return the maximum number of DP contact data stored for any node
            * -1,
            * -2, superelement flag
            * -3, master dof bit pattern
            * -4, active dof bit pattern
            * -5, solid model attachment
            * -6, pack nodal line parametric value
            * -7, constraint bit pattern
            * -8, force bit pattern
            * -9, body force bit pattern
            * -10, internal node flag
            * -11, orientation node flag =1 is =0 is not
            * -11, contact node flag <0
            * -12, constraint bit pattern (for ``DSYM``)
            * -13, if dof constraint written to file.k (for ``LSDYNA`` only)
            * -14, nodal coordinate system number (set by ``NROTATE``)
            * -101, pointer to node data record
            * -102, pointer to angle record
            * -103,
            * -104, pointer to attached couplings
            * -105, pointer to attacted constraint equations
            * -106, pointer to nodal stresses
            * -107, pointer to specified disp'S
            * -108, pointer to specified forces
            * -109, pointer to x/y/z record
            * -110,
            * -111,
            * -112, pointer to nodal temperatures
            * -113, pointer to nodal heat generations
            * -114,
            * -115, pointer to calculated displacements
            * -116

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        str, int, or float
            The returned value of ``ndinqr`` is based on setting of key.
        """
        return self.run(f"{pname} = ndinqr({node}, {key})", **kwargs)

    def elmiqr(self, ielem, key, pname="__tmpvar__", **kwargs):
        """Get information about an element.

        **Secondary Functions:**
        Set current element pointer to this element.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        ielem : int
            Element number.
            It should be zero for key=11, ``DB_NUMDEFINED``,
            ``DB_NUMSELECTED``, ``DB_MAXDEFINED``, ``DB_MAXRECLENG``,
            or 199

        key : int
            information flag.

            * DB_SELECTED - return select status:  (1)

              * 0 - element is undefined.
              * -1 - element is unselected.
              * 1 - element is selected.

            * ``DB_NUMDEFINED`` - return number of defined elements (12)
            * ``DB_NUMSELECTED`` - return number of selected elements (13)
            * ``DB_MAXDEFINED`` - return maximum element number used (14)
            * ``DB_MAXRECLENG`` - return maximum record length (15)
            * 2 - return length (int words)
            * 3 - return layer number (for cross reference files return number of entities)
            * 4 - return address of first data word
            * 5 - return length (in record type units)
            * 6 - return compressed record number.
            * 11 - return void percent (integer)
            * 16 - return location of next record (this increments the next record count)
            * 17 - pointer to start of index
            * 18 - return type of file.

              * 0 - integer
              * 1 - double precision
              * 2 - real
              * 3 - complex
              * 4 - character\\*8
              * 7 - index

            * 19 - return virtual type of file.

              * 0 - fixed length (4.4 form)
              * 1 - indexed variable length (layer data)
              * 2 - xref data tables
              * 3 - bitmap data (for 32 data item packed records)
              * 4 - data tables (three dimensional arrays)

            * 111 - return the maximum number of nodes stored for any element
            * 123 - return the maximum number of DP contact data stored for any element
            * -1 - material number ( ``= -EL_MAT``)
            * -2 - element type ( ``= -EL_TYPE``)
            * -3 - real constant number ( ``= -EL_REAL``)
            * -4 - element section ID number ( ``= -EL_SECT``)
            * -5 - coordinate system number ( ``= -EL_CSYS``)
            * -101 - pointer to element integers etc.

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            The returned value of ``elmiqr`` is based on setting of key.
        """
        return self.run(f"{pname} = elmiqr({ielem}, {key})", **kwargs)

    def kpinqr(self, knmi, key, pname="__tmpvar__", **kwargs):
        """Get information about a keypoints.

        **Secondary Functions:**
        Set current keypoints pointer to this keypoints.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        knmi : int
            Keypoints for inquire. may be 0 for key=11 thru 15.

        key : int
            Information flag.

            * 1 - return select

              * -1 - unselected
              * 0 - undefined
              * 1 - selected

            * 2 - return length (data units)
            * 3 - return layer number
              (for cross reference files return
              number of entities)
            * 4 - return address of first data word
            * 5 - return length (in record type units)
            * 6 - return compressed record number.
            * 11 - return void percent (integer)
            * 12 - return number of defined
            * 13 - return number of selected
            * 14 - return highest number defined
            * 15 - return maximum record length
              (data units)
            * 16 - return location of next record
              (this increments the next
              record count)
            * 18 - return type of file.

              * 0 - integer
              * 1 - double precision
              * 2 - real
              * 3 - complex
              * 4 - character\\*8
              * 7 - index

            * 19 - return virtual type of file.

              * 0 - fixed length (4.4 form)
              * 1 - indexed variable length
                (layer data)
              * 2 - xref data tables
              * 3 - bitmap data (for 32 data
                item packed records)
              * 4 - data tables (three
                dimensional arrays)

            * -1, material number
            * -2, type
            * -3, real number
            * -4, node number, if meshed
            * -5, pointer to attached point
            * -6, esys number
            * -7, element number, if meshed
            * -8, Hardpoint stuff
            * -9, area number associated with hardpoint
            * -10, line number associated with hardpoint
            * -11, Orientation kp flag
            * -12, local integer workspace
            * -101, pointer to keypoint data
            * -102, pointer to keypoint fluences
            * -103, pointer to keypoint moisture content
            * -104, pointer to keypoint voltage
            * -105, pointer to keypoint current density
            * -106, pointer to keypoint heat generations
            * -107, pointer to keypoint virtual displacements
            * -108, pointer to parameter data
            * -109, pointer to keypoint temperatures
            * -110, pointer to keypoint displacements
            * -111, pointer to keypoint forces
            * -112, pointer to line list

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            For key=1

            * 0 = ``knmi`` is undefined.
            * -1 = ``knmi`` is unselected.
            * 1 = ``knmi`` is selected.

            For key not equal to 1 returned data is based
            on setting of key.
        """
        return self.run(f"{pname} = kpinqr({knmi}, {key})", **kwargs)

    def lsinqr(self, line, key, pname="__tmpvar__", **kwargs):
        """Get information about a line segment.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        lnmi : int
            Line segment for inquire. It should be 0 for key=11 thru 15.

        key : float
            key as to information needed about the ``lnmi``.

            * 1, return select
            * 2, return length (data units)
            * 3,
            * 11, return void percent (integer)
            * 12, return number of defined
            * 13, return number of selected
            * 14, return highest number defined
            * 15, return maximum record length
              (data units)
            * 16, return location of next record
              (this increments the next
              record count)
            * 17, return next record from offset
            * -1, material number
            * -2, type
            * -3, real number
            * -4, number of nodes
            * -5, esys number
            * -6, number of elements
            * -7, pointer to line in foreign db
            * -8, # of elem divs in existing mesh
            * -9, keypoint 1
            * -10, keypoint 2
            * -11, color,translucency packed
            * -12, local integer workspace
              (used in delete with sweeps)
            * -13, orientation kpa
            * -14, orientation kpb
            * -15, section id
            * -16, # of elem divs for next mesh
            * -17, 0=hard / 1=soft ``NDIV``
            * -18, 0=hard / 1=soft ``SPACE``
            * -101, pointer to line segment data
            * -102,
            * -103,
            * -104,
            * -105, pointer to node list
            * -106,
            * -107, pointer to element list
            * -108, pointer to parameter data
            * -109,
            * -110, pointer to line convections
            * -111, pointer to line constraints
            * -112,
            * -113,
            * -114, pointer to area list
            * -115, pointer to sub-line list
            * -116, pointer to line pressures

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            For key=1

            * 0 = ``lnmi`` is undefined.
            * -1 = ``lnmi`` is unselected.
            * 1 = ``lnmi`` is selected.

            For key not equal to 1 returned data is based
            on setting of key.

        """
        return self.run(f"{pname} = lsinqr({line}, {key})", **kwargs)

    def arinqr(self, anmi, key, pname="__tmpvar__", **kwargs):
        """Get information about a area.

        **Secondary Functions:**
        Set current area pointer to this area.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        anmi : int
            Area for inquire. may be 0 for key=11 thru 15.

        key : float
            Key as to information needed about the ``anmi``.

            * 1, return select
            * 2, return length (data units)
            * 3,
            * 11, return void percent (integer)
            * 12, return number of defined
            * 13, return number of selected
            * 14, return highest number defined
            * 15, return maximum record length
              (data units)
            * 16, return next record
              (this increments the next record count)
            * -1, material
            * -2, type.
            * -3, real.
            * -4, number of nodes.
            * -5,
            * -6, number of elements.
            * -7, pointer to area in foreign db
            * -8, element shape.
            * -9, mid-node element key.
            * -10, element coordinate system.
            * -11, area constraint information.
            * 0 - no constraint on this area.
            * 1 - symmetry constraint.
            * 2 - anti-symmetry
            * 3 - both symmetry and anti-symmetry
            * -12, local integer workspace
            * -13,
            * -14,
            * -15, section
            * -16, color and translucency packed.
            * -101, pointer to area data
            * -102,
            * -103,
            * -104,
            * -105, pointer to node list.
            * -106, pointer to parameter data
            * -107, pointer to element list.
            * -108,
            * -109,
            * -110,
            * -111,
            * -112, pointer to line loop list
            * -113, pointer to volume xref
            * -114, pointer to sub-area list
            * -115, pointer to area presaraes
            * -116, pointer to area convections

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
          For key=1

          * 0 = ``anmi`` is undefined.
          * -1 = ``anmi`` is unselected.
          * 1 = ``anmi`` is selected.

          For key not equal to 1, the returned data is based
          on setting of key.
        """
        return self.run(f"{pname} = arinqr({anmi}, {key})", **kwargs)

    def vlinqr(self, vnmi, key, pname="__tmpvar__", **kwargs):
        """Get information about a volume.

        **Secondary Functions:**
        Set current volume pointer to this volume.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        vnmi : int
            Volume for inquire. may be 0 for key=11 thru 15.

        key : float
            Key as to information needed about the ``vnmi``.

            * 1, return select
            * 2, return length (data units)
            * 3,
            * 11, return void percent (integer)
            * 12, return number of defined
            * 13, return number of selected
            * 14, return highest number defined
            * 15, return maximum record length (data units)
            * 16, return next record
            * -1, material
            * -2, type.
            * -3, real.
            * -4, number of nodes.
            * -5, KZ1 - 1st kpt for elem Z
            * -6, number of elements.
            * -7, pointer to volume in foreign db
            * -8, element shape.
            * -9, (``section id``)\\*10 + 2
            * -10, element coordinate system.
            * -11, KZ2 - 2nd kpt for elem Z
            * -12, color and transparency packed
            * -101, pointer volume data file.
            * -102,
            * -103,
            * -104,
            * -105, pointer to node list.
            * -106, pointer to volume pvolmeter dat
            * -107, pointer to element list.
            * -108,
            * -109,
            * -110, pointer to sub-volume list
            * -111,
            * -112, pointer to area shell list

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            For key=1

            * 0 = ``vnmi`` is undefined.
            * -1 = ``vnmi`` is unselected.
            * 1 = ``vnmi`` is selected.

            For key ne 1 returned data is based
            on setting of key.
        """
        return self.run(f"{pname} = vlinqr({vnmi}, {key})", **kwargs)

    def rlinqr(self, nreal, key, pname="__tmpvar__", **kwargs):
        """Get information about a real constant set.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        nreal : int
            Real constant table number
            should be 0 for key=11, ``DB_NUMDEFINED``,
            ``DB_NUMSELECTED``, ``DB_MAXDEFINED``, and
            ``DB_MAXRECLENG``.

        key : int
            Information flag.

            * 5 - return number of values stored for ``nreal``.
              Return the REAL set width (number of fields)
            * ``DB_SELECTED`` - return select status

              * 0 - real constant table is undefined.
              * -1 - real constant table is unselected.
              * 1 - real constant table is selected

            * ``DB_NUMDEFINED`` - return number of defined real constant tables
            * ``DB_NUMSELECTED`` - return number of selected real constant tables
            * ``DB_MAXDEFINED`` - return highest real constant table defined
            * ``DB_MAXRECLENG`` - return maximum record length (dp words)

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            The returned value of ``rlinqr`` is based on setting of key.

        """
        return self.run(f"{pname} = rlinqr({nreal}, {key})", **kwargs)

    def gapiqr(self, ngap, key, pname="__tmpvar__", **kwargs):
        """Get information about a dynamic gap set.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        ngap : int
            gap number for inquire (must be zero for now).

        key : int
            key as to the information needed

            * 1, return select
            * 2, return length (data units)
            * 3,
            * 11, return void percent (integer)
            * 12, return number of defined
            * 13, return number of selected
            * 14, return highest number defined
            * 15, return maximum record length (data units)

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            Returned data is based on setting of key.
        """
        return self.run(f"{pname} = gapiqr({ngap}, {key})", **kwargs)

    def masiqr(self, node, key, pname="__tmpvar__", **kwargs):
        """Get information about masters nodes.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        node : int
            Node number for inquire (must be zero for now).

        key : int
            Key as to the information needed

            * 1, return select
            * 2, return length (data units)
            * 3,
            * 11, return void percent (integer)
            * 12, return number of defined
            * 13, return number of selected
            * 14, return highest number defined
            * 15, return maximum record length (data units)

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int
            Returned data is based on setting of key.
        """
        return self.run(f"{pname} = masiqr({node}, {key})", **kwargs)

    def ceinqr(self, nce, key, pname="__tmpvar__", **kwargs):
        """Get information about a constraint equation set.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        nce : int
            Constraint equation number

        key : int
            Inquiry key.
            It should be zero for key=11, ``DB_NUMDEFINED``,
            ``DB_NUMSELECTED``, ``DB_MAXDEFINED``, and
            ``DB_MAXRECLENG``

            * ``DB_SELECTED`` - return select status

              * 1 - equation is selected
              * 0 - equation is undefined
              * -1 - equation is unselected

            * ``DB_NUMDEFINED`` - return number of defined constraint
              equations.
            * ``DB_NUMSELECTED`` - return number of selected constraint
              equations.
            * ``DB_MAXDEFINED`` - return number of highest numbered
              constraint equation defined.
            * ``DB_MAXRECLENG`` - return length of longest constraint
              equation set (max record length)
            * 2 - return length (data units)
            * 3 - return layer number
            * 4 - address of first data word
            * 5 - return number of values stored for nce
            * 11 - return void percent (integer)
            * 16 - return location of next record
            * ``CE_NONLINEAR`` - return 1 if CE is nonlinear
            * ``CE_ELEMNUMBER`` - return associated element number

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            The returned value of ``ceinqr`` is based on setting of key.
        """
        return self.run(f"{pname} = ceinqr({nce}, {key})", **kwargs)

    def cpinqr(self, ncp, key, pname="__tmpvar__", **kwargs):
        """Get information about a coupled set.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        ncp : int
            Coupled set number.

        key : int
            Inquiry key. It should be zero for key=11, ``DB_NUMDEFINED``,
            ``DB_NUMSELECTED``, ``DB_MAXDEFINED``, and ``DB_MAXRECLENG``

            * ``DB_SELECTED`` - return select status

              * 1 - coupled set is selected
              * 0 - coupled set in undefined
              * -1 - coupled set in unselected

            * ``DB_NUMDEFINED`` - return number of defined coupled sets
            * ``DB_NUMSELECTED`` - return number of selected coupled sets
            * ``DB_MAXDEFINED`` - return the number of the highest numbered
              coupled set
            * ``DB_MAXRECLENG`` - return length of largest coupled set record
              (max record length)
            * 2 - return length (data units)
            * 3 - return layer number
            * 4 - return address of first data word
            * 5 - return number of values stored for ncp
            * 11 - return void percent (integer)
            * 16 - return location of next record
            * -1 - return master node for this eqn (this is currently only used by solution DB object)

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            The returned value of ``cpinqr`` is based on setting of key.

        """
        return self.run(f"{pname} = cpinqr({ncp}, {key})", **kwargs)

    def csyiqr(self, ncsy, key, pname="__tmpvar__", **kwargs):
        """Get information about a coordinate system.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        ncsy : int
            Coordinate system reference number
            should be zero for key= ``DB_NUMDEFINED``
            or ``DB_MAXDEFINED``

        key : int
            Information flag.

            * ``DB_SELECTED`` - return status:

              * 0 - coordinate system is not defined
              * -1 - coordinate system is not selected
              * 1 - coordinate system is selected

            * ``DB_NUMDEFINED`` - number of defined coordinate systems
            * ``DB_MAXDEFINED`` - maximum coordinate system reference
              number used.

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            The returned value of ``csyiqr`` is based on setting of key.
        """
        return self.run(f"{pname} = csyiqr({ncsy}, {key})", **kwargs)

    def etyiqr(self, itype, key, pname="__tmpvar__", **kwargs):
        """Get information about an element type.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        itype : int
            Element type number.
            It should be 0 for key=11, ``DB_NUMDEFINED``,
            ``DB_NUMSELECTED``, ``DB_MAXDEFINED``, and
            ``DB_MAXRECLENG``

        key : int
            Information flag.

            * ``DB_SELECTED`` - return select status:

              * 0 - element type is undefined.
              * -1 - element type is unselected.
              * 1 - element type is selected.

            * ``DB_NUMDEFINED`` - return number of defined element types
            * ``DB_NUMSELECTED`` - return number of selected element types
            * ``DB_MAXDEFINED`` - return highest element type number defined
            * ``DB_MAXRECLENG`` - return maximum record length (int words)
            * ``-n``- return element characteristic n from ``etycom`` for element
              type itype.
              ``n`` is correlated to the parameter names in ``echprm``.
              see ``elccmt`` for definitions of element characteristics.

              .. note:: This will not overwrite the current setting of
                        ``etycom``.

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            The returned value of ``etyiqr`` is based on setting of key.
        """
        return self.run(f"{pname} = etyiqr({itype}, {key})", **kwargs)

    def foriqr(self, node, key, pname="__tmpvar__", **kwargs):
        """Get information about nodal loads.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        node : int
            Number of node being inquired about.
            should be 0 for key = ``DB_MAXDEFINED`` or
            ``DB_NUMDEFINED``.

        key : float
            Key as to information needed

            * 1 - return force mask for node
            * ``DB_MAXDEFINED`` - return number of nodal loadings in model.
            * ``DB_NUMDEFINED`` - return number of nodal loadings in model.

            .. note:: Both ``DB_MAXDEFINED`` and ``DB_NUMDEFINED``, produce the
                     same functionality.

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            The returned value of ``foriqr`` is based on setting of key.
        """
        return self.run(f"{pname} = foriqr({node}, {key})", **kwargs)

    def sectinqr(self, nsect, key, pname="__tmpvar__", **kwargs):
        """Get information about a section id set.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        nsect : int
            Section id table number
            should be 0 for key=11, ``DB_NUMDEFINED``,
            ``DB_NUMSELECTED``, ``DB_MAXDEFINED``, and
            ``DB_MAXRECLENG``.

        key : int
            Information flag.

            * ``DB_SELECTED`` - return select status

              * 0 - ection id table is undefined.
              * -1 - section id table is unselected.
              * 1 - section id table is selected

            * ``DB_NUMDEFINED`` - return number of defined section id tables
            * ``DB_NUMSELECTED`` - return number of selected section id tables
            * ``DB_MAXDEFINED`` - return highest section id table defined
            * ``DB_MAXRECLENG`` - return maximum record length (dp words)
            * 2 - return length (dp words)
            * 3 - return layer number (for cross reference files return number
              of entities)
            * 4 - return address of first data word
            * 5 - return length (in record type units)
            * 6 - return compressed record number.
            * 11 - return void percent (integer)
            * 16 - return location of next record (this increments the next
              record count)
            * 18 - return type of file.

              * 0 - integer
              * 1 - double precision
              * 2 - real
              * 3 - complex
              * 4 - character*8
              * 7 - index

            * 19 - return virtual type of file.

              * 0 - fixed length (4.4 form)
              * 1 - indexed variable length (layer data)
              * 2 - xref data tables
              * 3 - bitmap data (for 32 data item packed records)
              * 4 - data tables (three dimensional arrays)

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            The returned value of sectinqr is based on setting of key.
        """
        return self.run(f"{pname} = sectinqr({nsect}, {key})", **kwargs)

    def mpinqr(self, mat, iprop, key, pname="__tmpvar__", **kwargs):
        """Get information about a material property.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        mat : int
            Material number
            should be 0 for key=11, ``DB_NUMDEFINED(12)``,
            ``DB_MAXDEFINED(14)``, and ``DB_MAXRECLENG(15)``.

        iprop  :  int
            Property reference number (See Table-1_ in the Notes section).

            If iprop = 0, test for existence of any material property with this
            material number (with key = ``DB_SELECTED(1)``).

        key : int
            Key as to the information needed about material property.

            * ``DB_SELECTED(1)`` - return select status:

              * 0 - material prop is undefined.
              * 1 - material prop is selected.

            * ``DB_NUMDEFINED(12)`` - number of defined material properties
            * ``DB_MAXDEFINED(14)`` - highest material property number defined
            * ``DB_MAXRECLENG(15)`` - maximum record length (dp words)
            * 2 - return length (dp words)
            * 3 - return number of temp. values
            * 11 - return void percent (integer)

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            Returned value of ``mpinqr`` is based on setting of key.

        Notes
        -----

        The material properties are obtained from the :meth:`MP <ansys.mapdl.core.Mapdl.mp>` command labels,
        which are detailed below:

        **MP command labels**

        .. _Table-1:

        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | Property | Key | Property | Key | Property | Key | Property | Key | Property | Key | Property | Key |
        +==========+=====+==========+=====+==========+=====+==========+=====+==========+=====+==========+=====+
        | EX       | 1   | DAMP     | 15  | PRYZ     | 29  | SBKX     | 43  | HGLS     | 57  | RH       | 71  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | EY       | 2   | KXX      | 16  | PRXZ     | 30  | SBKY     | 44  | BVIS     | 58  | DXX      | 72  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | EZ       | 3   | KYY      | 17  | MURX     | 31  | SBKZ     | 45  | QRAT     | 59  | DYY      | 73  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | NUXY     | 4   | KZZ      | 18  | MURY     | 32  | SONC     | 46  | REFT     | 60  | DZZ      | 74  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | NUYZ     | 5   | RSVX     | 19  | MURZ     | 33  | DMPS     | 47  | CTEX     | 61  | BETX     | 75  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | NUXZ     | 6   | RSVY     | 20  | PERX     | 34  | ELIM     | 48  | CTEY     | 62  | BETY     | 76  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | GXY      | 7   | RSVZ     | 21  | PERY     | 35  | USR1     | 49  | CTEZ     | 63  | BETZ     | 77  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | GYZ      | 8   |          | 22  | PERZ     | 36  | USR2     | 50  | THSX     | 64  | CSAT     | 78  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | GXZ      | 9   | HF       | 23  | MGXX     | 37  | USR3     | 51  | THSY     | 65  | CREF     | 79  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | ALPX     | 10  | VISC     | 24  | MGYY     | 38  | USR4     | 51  | THSZ     | 66  | CVH      | 80  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | ALPY     | 11  | EMIS     | 25  | MGZZ     | 39  | FLUI     | 53  | DMPR     | 67  | UMID     | 81  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | ALPZ     | 12  | ENTH     | 26  | EGXX     | 40  | ORTH     | 54  | LSSM     | 68  | UVID     | 82  |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | DENS     | 13  | LSST     | 27  | EGYY     | 41  | CABL     | 55  | BETD     | 69  |          |     |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+
        | MU       | 14  | PRXY     | 28  | EGZZ     | 42  | RIGI     | 56  | ALPD     | 70  |          |     |
        +----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+----------+-----+

        see :meth:`TB <ansys.mapdl.core.Mapdl.tb>` command for more information.

        """
        return self.run(f"{pname} = mpinqr({mat}, {iprop}, {key})", **kwargs)

    def dget(self, node, idf, kcmplx, pname="__tmpvar__", **kwargs):
        """Get a constraint from the data base.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        node : int
            Node number

        idf : int
            Pointer to the dof (1-32)

            * 1 = ux,
            * 2 = uy
            * 3 = uz
            * 4 = rotx
            * 5 = roty
            * 6 = rotz
            * 7 = ax
            * 8 = ay
            * 9 = az
            * 10 = vx
            * 11 = vy
            * 12 = vz
            * 13-18 = spares
            * 19 = pres
            * 20 = temp
            * 21 = volt
            * 22 = mag
            * 23 = enke
            * 24 = ends
            * 25 = emf
            * 26 = curr
            * 27-32 = SP01-SP06

        kcmplx : int

            * 0 = real
            * 1 = imaginary

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        float
            Constraint value (High number if undefined)
        """
        return self.run(f"{pname} = dget({node}, {idf}, {kcmplx})", **kwargs)

    def fget(self, node, idf, kcmplx, pname="__tmpvar__", **kwargs):
        """Get a force load from the data base.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        node : int
            Node number

        idf : int
            Pointer to the dof (1-32)

            * 1 = ux
            * 2 = uy
            * 3 = uz
            * 4 = rotx
            * 5 = roty
            * 6 = rotz
            * 7 = ax
            * 8 = ay
            * 9 = az
            * 10 = vx
            * 11 = vy
            * 12 = vz
            * 13-18 = spares
            * 19 = pres
            * 20 = temp
            * 21 = volt
            * 22 = mag
            * 23 = enke
            * 24 = ends
            * 25 = emf
            * 26 = curr
            * 27-32 = spares

        kcmplx : int

            * 0 = real
            * 1 = imaginary

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        float
            Force value (high number if undefined)
        """
        return self.run(f"{pname} = fget({node}, {idf}, {kcmplx})", **kwargs)

    def erinqr(self, key, pname="__tmpvar__", **kwargs):
        """Obtain information from common errors.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        Parameters
        ----------
        key : int
            Item to be returned.

            * 1 = keyerr (``ER_ERRORFLAG``)

              Master error flag.

            * 2 = errfil (``ER_ERRORFILE``)

              Errors file unit number.

            * 3 = numnot (``ER_NUMNOTE``)

              Total number of notes displayed.

            * 4 = numwrn (``ER_NUMWARNING``)

              Total number of warnings displayed.

            * 5 = numerr (``ER_NUMERROR``)

              Total number of errors displayed.

            * 6 = numfat (``ER_NUMFATAL``)

              Total number of fatals displayed.

            * 7 = maxmsg (``ER_MAXMESSAGE``)

              Max allowed number of displayed messages before abort.

            * 8 = lvlerr (``ER_ERRORLEVEL``)

              Used basically in solution (from ``cnvr`` command.)

              * -1 = do not set keyerr for notes/errors/warnings.
              * -2 = same as -1 but do not display message either.

            * 9 = mxpcmd (``ER_MAXCOMMAND``)

              Maximum number of messages allowed per command.

            * 10 = nercmd (``ER_NUMCOMMAND``)

              Number of messages displayed for any one command.

            * 11 = nertim (``ER_UICLEAR``)

              Key as to how message cleared from u/i pop-up (only for "info"
              calls)

              * -1 = message is timed before removal
              * 0 = message needs pick or keyboard before removal
              * 1 = message stays up until replaced by another message

            * 12 = nomore (``ER_NOMOREMSG``)

              Display no more messages

              * 0 = display messages
              * 1 = display discontinue message and stop displaying

            * 13 = eropen (``ER_FILEOPEN``)

              * 0 = errors file is closed
              * 1 = errors file is opened

            * 14 = ikserr (``ER_INTERERROR``)

              * 0 = if interactive do not set keyerr.
              * -1 = if interactive set keyerr (used by mesher and tessalation)

            * 15 = kystat (``ER_KEYOPTTEST``)

              Flag to bypass keyopt tests in the elcxx routines associated with
              status/panel info inquiries.

              * 0 = do not bypass keyopt tests
              * 1 = perform all keyopt tests also flag to bypass setting of
                ``_STATUS`` upon resume

            * 16 = mxr4r5 (``ER_MIXEDREV``)

              Mixed rev4-rev5 input logic (``*do``, ``*if`` , ``*go`` , ``*if-go``)

              * 1 = rev5 found (``*do``, ``*fi-then-*endif``)
              * 2 = rev4 found (``*go``, ``:xxx`` , ``*if``, ...., ``:xxx``)
              * 3 = warning printed. do not issue any more.

            * 17 = mshkey (``ER_MESHING``)

              CPU intensive meshing etc.
              This will cause ``nertim (11)`` to be set to ``-1`` for "notes", ``1`` for
              "warnings",and ``0`` for "errors".
              Checking of this key is done in ``anserr``.

              * 0 = not meshing or cpu intensive
              * 1 = yes, meshing or cpu intensive

            * 18 = syerro (18)

              Systop error code. read by anserr if set.

            * 19 = opterr (``ER_OPTLOOPING``)

              * 0 = no error in main ansys during opt looping
              * 1 = an error has happened in main ansys during opt looping

            * 20 = flowrn (20)

              Flag used by "floqa" as to list ``floqa.ans``.

              * 0 = list ``floqa.ans``
              * 1 = ``floqa.ans`` has been listed. do not list again.

            * 22 = noreport (22)

              Used in GUI for turning off errors due to strsub calls.

              * 0 = process errors as usual
              * 1 = do **NOT** report errors

            * 23 = pdserr (``ER_PDSLOOPING``)

              * 0 = no error in main ansys during pds looping
              * 1 = an error has happened in main ansys during pds looping

            * 24 = mxpcmdw (24)

              Number of messages written to file.err for any command

              * 0 = write all errors to file.err
              * 1 = only write displayed errors to file.err

            * 25 = kystop

              No information is provided.

            * 26 = icloads (26)

              Key to forbid the ``iclist`` command from listing solution data
              instead of the input data.

              * 0 = ``iclist`` is OK
              * 1 = do not permit ``iclist``

            * 27 = ifkey error (27)

              key on whether or not to abort during :meth:`/input <ansys.mapdl.core.Mapdl.input>` on error.

              * 0 = do not abort
              * 1 = abort

            * 28 = intrupt (``ER_INTERRUPT``)

              Interrupt button, so executable returns no error.

            * spare - spare integer variables

        pname : str
            Name of the variable where the queried value is stored.

        **kwargs
            Extra arguments to be passed to :meth:`Mapdl.run <ansys.mapdl.core.Mapdl.run>`.

        Returns
        -------
        int or str
            Value corresponding to key.

        """
        return self.run(f"{pname} = erinqr({key})", **kwargs)

    def wrinqr(self, key, pname="__tmpvar__", **kwargs):
        """Obtain information about output.

        .. warning:: **DISCLAIMER**: This function is un-documented in the
                   official ANSYS Command Reference Guide.
                   Hence its support is limited and it use is not encouraged.
                   **Please use it with caution.**

        .. warning:: Caution: the following variables are "saved/resumed".
                    key=``WR_COLINTER`` thru ``WR_SUPCOLMAX`` in "wrinqr/wrinfo".

        Parameters
        ----------
        key : int
            Key.

            * 1 - kprint (WR_PRINT)
              Print flag.

              * 0 - no output
              * 1 - print

            * 2 - outfil (WR_OUTPUT)
              Current output unit number(iott).
            * 4 - frstot (WR_MASTEROUT)
              Master output file.
            * 5 - intcol (WR_COLINTER)
              Interactive columns per page.
            * 6 - batcol (WR_COLBATCH)
              Batch columns per page.
            * 7 - intlin (WR_LINEINTER)
              Interactive lines per page.
            * 8 - batlin (WR_LINEBATCH)
              Batch lines per page.
            * 9 - CommaSep (WR_COMMASEP)
              1 for comma separated output.
            * 11 - chrper (WR_CHARITEM)
              Characters per output item.
            * 12 - chrdec (WR_CHARDECIMAL)
              Characters past decimal.
            * 13 - chrint (WR_CHARINTEGER)
              Characters in leading integer.
            * 14 - chrtyp (WR_CHARTYPE)

              * 1 - using E format in output.
              * 2 - using F format in output.
              * 3 - using G format in output.

            * 15 - (WR_TEMPLINEBATCH)
              Undocumented (50 default).
            * 16 - keyhed (WR_SUPTITLE)
              Tlabel suppress key.
            * 17 - keytit (WR_SUPSUBTITLE)
              Subtitle suppress key.
            * 18 - keyid (WR_SUPLSITER)
              Ls,iter id suppress key.
            * 19 - keynot (WR_NOTELINE)
              Note line suppress key.
            * 20 - keylab (WR_SUPCOLHEADER)
               column header suppress key.
            * 21 - keysum (WR_SUPCOLMAX)
              Column maximum suppress key.
            * 22 - ListOpt (WR_LISTOPT)
              ListOpt from /output command.

        Returns
        -------
        int or str
            The value corresponding to key.

        """
        return self.run(f"{pname} = wrinqr({key})", **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class Morphing:
    def morph(
        self,
        option="",
        remeshopt="",
        elemset="",
        armax="",
        voch="",
        arch="",
        step="",
        time="",
        stropt="",
        **kwargs,
    ):
        """Specifies morphing and remeshing controls.

        APDL Command: MORPH

        Parameters
        ----------
        option
            OFF - Turns off morphing for field elements (default).

            ON - Turns on morphing for field elements.

        remeshopt
            OFF - Do not remesh (default).

            ON - Remesh when element qualities fall below values
                 specified by ARMAX, VOCH, or ARCH as explained
                 below. Valid only when Option is ON.

        elemset
            ALL - Remesh all selected elements if the quality of the
                  worst defined element falls below any quality
                  requirement (default when Remeshopt = ON).

            CompName - Specify a component name, up to 32
                        characters. All elements included in this
                        component name are remeshed if the quality of
                        the worst element falls below any quality
                        requirement.

        armax
            The maximum allowable element generalized aspect ratio. Defaults to
            5.

        voch
            The maximum allowable change of element size (area or volume).
            Defaults to 3.

        arch
            The maximum allowable element aspect ratio change.  Defaults to 3.

        step
            The frequency of element quality checking, based on time steps. A
            quality check takes place at the intervals defined by STEP.
            Defaults to 1 (quality check at every step).

        time
            A quality check takes place at the time point specified. Defaults
            to -1 (a quality check at every time point).

        stropt
            NO

            NO - There are no structural elements in the model (default).

            YES - There are no structural elements in the model and the morphing happens after
                  the structural solution.

        Notes
        -----
        MORPH is applicable to any non-structural field analysis (not including
        fluid elements). It activates displacement degrees of freedom for non-
        structural elements so that boundary conditions may be placed on the
        field mesh to constrain the movement of the non-structural mesh during
        morphing. It morphs the non-structural mesh using displacements
        transferred at the surface interface between the structural field and
        the non-structural field.  The displacements of non-structural elements
        are mesh displacements to avoid mesh distortion, but have no physical
        meaning except at the interface. MORPH does not support surface, link,
        or shell elements, or any element shape other than triangles, quads,
        tets, and bricks. Morphed fields must be in the global Cartesian system
        (CSYS = 0).

        After each remesh, new databases and results files are written with the
        extensions .rth0n and .db0n, where n is the remesh file number
        (FieldName.rth01, FieldName.rth02, ... and FieldName.db01,
        FieldName.db02, etc.).   The original database file is FieldName.dbo.
        The FieldName.db01, FieldName.db02, etc. files have elements that are
        detached from the solid model.

        Remeshing has the following restrictions:

        Valid only for the electrostatic elements (PLANE121, SOLID122, and
        SOLID123)

        Limited to triangle (2-D) and tetrahedral (3-D) options of these
        elements

         Valid only for the MFS solver

        No body loads allowed in the interior nodes of the remeshing domain

        Nodes on the boundary cannot be remeshed; remeshing will not work if
        morphing failed on the surface nodes

        Not suitable for extreme area or volume changes

        This command is also valid in SOLUTION.
        """
        command = f"MORPH,{option},,{remeshopt},{elemset},{armax},{voch},{arch},{step},{time},{stropt}"
        return self.run(command, **kwargs)

    def damorph(self, area="", xline="", rmshky="", **kwargs):
        """Move nodes in selected areas to conform to structural displacements.

        APDL Command: DAMORPH

        Parameters
        ----------
        area
            Non-structural area to which mesh movement (morph) applies.  If
            ALL, apply morphing to all selected areas [ASEL].  If AREA = P,
            graphical picking is enabled.  A component may be substituted for
            AREA.

        xline
            Lines to be excluded from morphing.  If ALL, exclude all selected
            lines [LSEL] from morphing.  If XLINE = P, graphical picking is
            enabled.  A component may be substituted for XLINE.  If XLINE is
            blank (default), allow morphing of nodes attached to lines of the
            selected areas (AREA) which are not shared by unselected areas.
            See Notes for clarification.

        rmshky
            Remesh flag option:

            0 - Remesh the selected non-structural areas only if mesh morphing fails.

            1 - Remesh the selected non-structural areas and bypass mesh morphing.

            2 - Perform mesh morphing only and do not remesh.

        Notes
        -----
        The selected areas should include only non-structural regions adjacent
        to structural regions. DAMORPH will morph the non-structural areas to
        coincide with the deflections of the structural regions.

        Nodes in the structural regions move in accordance with computed
        displacements. Displacements from a structural analysis must be in the
        database prior to issuing DAMORPH.

        By default, nodes attached to lines can move along the lines, or off
        the lines (if a line is interior to the selected areas). You can use
        XLINE to restrain nodes on certain lines.

        By default (RMSHKEY = 0), DAMORPH will remesh the selected non-
        structural areas entirely if a satisfactory morphed mesh cannot be
        provided.

        If boundary conditions and loads are applied directly to nodes and
        elements, the DAMORPH command requires that these be removed before
        remeshing can take place.

        Exercise care with initial conditions defined by the IC command. Before
        a structural analysis is performed for a sequentially coupled analysis,
        the DAMORPH command requires that initial conditions be removed from
        all null element type nodes in the non-structural regions. Use ICDELE
        to delete the initial conditions.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"DAMORPH,{area},{xline},{rmshky}"
        return self.run(command, **kwargs)

    def demorph(self, elem="", dimn="", rmshky="", **kwargs):
        """Move nodes in selected elements to conform to structural displacements.

        APDL Command: DEMORPH

        Parameters
        ----------
        elem
            Non-structural elements to which mesh movement (morph) applies.
            If ALL, apply morphing to all selected elements [ESEL]. If ELEM =
            P, graphical picking is enabled.  A component may be substituted
            for ELEM.

        dimn
            Problem dimensionality.  Use "2" for a 2-D problem and "3" for a
            3-D problem (no default).

        rmshky
            Remesh flag option:

            0 - Remesh the selected non-structural regions only if mesh morphing fails.

            1 - Remesh the selected non-structural regions and bypass mesh morphing.

            2 - Perform mesh morphing only and do not remesh.

        Notes
        -----
        The selected elements should include only non-structural regions
        adjacent to structural regions. The exterior nodes of the selected
        elements will usually be on the boundary of the region which will have
        node positions displaced. For DIMN = 2, elements must lie on a flat
        plane. The DEMORPH command requires a single domain grouping of
        elements be provided (multiple domains of elements are not permitted).
        Exterior nodes will be assumed fixed (no nodes will be morphed) unless
        they coincide with structural nodes having nonzero displacements.

        Nodes in the structural regions move in accordance with computed
        displacements. Displacements from a structural analysis must be in the
        database prior to issuing DEMORPH.

        By default (RMSHKY = 0), DEMORPH will remesh the selected non-
        structural regions entirely if a satisfactory morphed mesh cannot be
        provided.

        If boundary conditions and loads are applied directly to nodes and
        elements, the DEMORPH command requires that these be removed before
        remeshing can take place.

        Exercise care with initial conditions defined by the IC command. Before
        a structural analysis is performed for a sequentially coupled analysis,
        the DEMORPH command requires that initial conditions be removed from
        all null element type nodes in the non-structural regions. Use ICDELE
        to delete the initial conditions.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"DEMORPH,{elem},{dimn},{rmshky}"
        return self.run(command, **kwargs)

    def dvmorph(self, volu="", xarea="", rmshky="", **kwargs):
        """Move nodes in selected volumes to conform to structural displacements.

        APDL Command: DVMORPH

        Parameters
        ----------
        volu
            Non-structural volume to which mesh movement (morph) applies.  If
            ALL, apply morphing to all selected volumes [VSEL]. If VOLU  = P,
            graphical picking is enabled. A component may be substituted for
            VOLU.

        xarea
            Areas to be excluded from morphing. If ALL, exclude all selected
            areas [ASEL].  If XAREA = P, graphical picking is enabled. A
            component may be substituted for XAREA. If XAREA is blank
            (default), allow morphing of nodes attached to areas of the
            selected volumes (VOLU) which are not shared by unselected volumes.
            (See Notes for clarification).

        rmshky
            Remesh flag option:

            0 - Remesh the selected non-structural volumes only if mesh morphing fails.

            1 - Remesh the selected non-structural volumes and bypass mesh morphing.

            2 - Perform mesh morphing only and do not remesh.

        Notes
        -----
        The selected volumes should include only non-structural regions
        adjacent to structural regions. DVMORPH will morph the non-structural
        volumes to coincide with the deflections of the structural regions.

        Nodes in the structural regions move in accordance with computed
        displacements. Displacements from a structural analysis must be in the
        database prior to issuing DVMORPH.

        By default, nodes attached to areas can move along the areas. You can
        use XAREA to restrain nodes on certain areas.

        By default (RMSHKY = 0), DVMORPH will remesh the selected non-
        structural volumes entirely if a satisfactory morphed mesh cannot be
        provided.

        If boundary conditions and loads are applied directly to nodes and
        elements, the DVMORPH command requires that these be removed before
        remeshing can take place.

        Exercise care with initial conditions defined by the IC command. Before
        a structural analysis is performed for a sequentially coupled analysis,
        the DVMORPH command requires that initial conditions be removed from
        all null element type nodes in the non-structural regions. Use ICDELE
        to delete the initial conditions.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"DVMORPH,{volu},{xarea},{rmshky}"
        return self.run(command, **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class MultiFieldSolverDefinitionCommands:
    def mfcmmand(self, fnumb="", fname="", ext="", **kwargs):
        """Captures field solution options in a command file.

        APDL Command: MFCMMAND

        Parameters
        ----------
        fnumb
            Field number specified by the MFELEM command.

        fname
            Command file name specified for the field number. Defaults to field
            "FNUMB".

        ext
            Extension for Fname. Defaults to .cmd.

        Notes
        -----
        All relevant solution option commands for the specified field are
        written to a file with the extension .cmd. Refer to the commands in the
        following tables in the Command Reference: Analysis Options, Nonlinear
        Options, Dynamic Options, and Load Step Options.

        This command is also valid in PREP7.

        See Multi-field Commands in the Coupled-Field Analysis Guide for a list
        of all ANSYS Multi-field solver commands and their availability for MFS
        and MFX analyses.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"MFCMMAND,{fnumb},{fname},{ext}"
        return self.run(command, **kwargs)

    def mfelem(
        self,
        fnumb="",
        itype1="",
        itype2="",
        itype3="",
        itype4="",
        itype5="",
        itype6="",
        itype7="",
        itype8="",
        itype9="",
        itype10="",
        **kwargs,
    ):
        """Defines a field by grouping element types.

        APDL Command: MFELEM

        Parameters
        ----------
        fnumb
            Field number for a group of element types.

        itype1, itype2, itype3, . . . , itype10
            Element types defined by the ET command.

        Notes
        -----
        You can define up to ten element types per field.

        Define only element types that contain elements in the field. Do not
        include MESH200 because it is a "mesh-only" element that does not
        contribute to the solution.

        This command is also valid in PREP7.

        See Multi-field Commands in the Coupled-Field Analysis Guide for a list
        of all ANSYS Multi-field solver commands and their availability for MFS
        and MFX analyses.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"MFELEM,{fnumb},{itype1},{itype2},{itype3},{itype4},{itype5},{itype6},{itype7},{itype8},{itype9},{itype10}"
        return self.run(command, **kwargs)

    def mfem(
        self,
        fnumb="",
        itype1="",
        itype2="",
        itype3="",
        itype4="",
        itype5="",
        itype6="",
        itype7="",
        itype8="",
        itype9="",
        itype10="",
        **kwargs,
    ):
        """Add more element types to a previously defined field number.

        APDL Command: MFEM

        Parameters
        ----------
        fnumb
            Existing field number defined by the MFELEM command.

        itype1, itype2, itype3, . . . , itype10
            Element types defined by the ET command.

        Notes
        -----
        You can add up to ten element types per MFEM command. This command
        should not be used after an initial solution.

        This command is also valid in PREP7.

        See Multi-field Commands in the Coupled-Field Analysis Guide for a list
        of all ANSYS Multi-field solver commands and their availability for MFS
        and MFX analyses.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"MFEM,{fnumb},{itype1},{itype2},{itype3},{itype4},{itype5},{itype6},{itype7},{itype8},{itype9},{itype10}"
        return self.run(command, **kwargs)

    def mfexter(
        self,
        fnumb1="",
        fnumb2="",
        fnumb3="",
        fnumb4="",
        fnumb5="",
        fnumb6="",
        fnumb7="",
        fnumb8="",
        fnumb9="",
        fnumb10="",
        fnumb11="",
        fnumb12="",
        fnumb13="",
        fnumb14="",
        fnumb15="",
        fnumb16="",
        fnumb17="",
        fnumb18="",
        fnumb19="",
        fnumb20="",
        **kwargs,
    ):
        """Defines external fields for an ANSYS Multi-field solver analysis.

        APDL Command: MFEXTER

        Parameters
        ----------
        fnumb1, fnumb2, fnumb3, . . . , fnumb20
            External field numbers defined by the MFELEM command.

        Notes
        -----
        This command specifies external field numbers to be used for load
        transfer in an ANSYS Multi-field solver analysis. Use the MFIMPORT
        command to import the external fields.

        Use the MFELEM command to specify external field numbers. Use the
        MFORDER command to specify the solution order for the external fields.

        You can define a maximum of 20 fields.

        This command is also valid in PREP7.

        See Multi-field Commands in the Coupled-Field Analysis Guide for a list
        of all ANSYS Multi-field solver commands and their availability for MFS
        and MFX analyses.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"MFEXTER,{fnumb1},{fnumb2},{fnumb3},{fnumb4},{fnumb5},{fnumb6},{fnumb7},{fnumb8},{fnumb9},{fnumb10},{fnumb11},{fnumb12},{fnumb13},{fnumb14},{fnumb15},{fnumb16},{fnumb17},{fnumb18},{fnumb19},{fnumb20}"
        return self.run(command, **kwargs)

    def mffname(self, fnumb="", fname="", **kwargs):
        """Specifies a file name for a field in an ANSYS Multi-field solver

        APDL Command: MFFNAME
        analysis.

        Parameters
        ----------
        fnumb
            Field number specified by the MFELEM command.

        fname
            File name. Defaults to field "FNUMB".

        Notes
        -----
        All files created for the field will have this file name with the
        appropriate extensions.

        This command is also valid in PREP7.

        See Multi-field Commands in the Coupled-Field Analysis Guide for a list
        of all ANSYS Multi-field solver commands and their availability for MFS
        and MFX analyses.

        Distributed ANSYS Restriction: This command is not supported in
        Distributed ANSYS.
        """
        command = f"MFFNAME,{fnumb},{fname}"
        return self.run(command, **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from typing import Optional, Union
import warnings

from ansys.mapdl.core.mapdl_types import MapdlInt


class Style:
    def cplane(self, key="", **kwargs):
        """Specifies the cutting plane for section and capped displays.

        APDL Command: /CPLANE

        Parameters
        ----------
        key
            Specifies the cutting plane:

            0 - Cutting plane is normal to the viewing vector [/VIEW] and passes through the
                focus point [/FOCUS] (default).

            1 - The working plane [WPLANE] is the cutting plane.

        Notes
        -----
        Defines the cutting plane to be used for section and capped displays
        [/TYPE,,(1, 5, or 7)].

        This command is valid in any processor.
        """
        command = f"/CPLANE,{key}"
        return self.run(command, **kwargs)

    def ctype(self, key="", dotd="", dots="", dshp="", tlen="", **kwargs):
        """Specifies the type of contour display.

        APDL Command: /CTYPE

        Parameters
        ----------
        key
            Type of display:

            0 - Standard contour display.

            1 - Isosurface display.

            2 - Particle gradient display.

            3 - Gradient triad display.

        dotd
            Maximum dot density for particle gradient display (KEY = 2).
            Density is expressed as dots per screen width (defaults to 30).

        dots
            Dot size for particle gradient display (KEY = 2).  Size is
            expressed as a fraction of the screen width (defaults to 0.0
            (single dot width)).

        dshp
            Spherical dot shape precision for particle gradient display (KEY =
            2).  (3-D options are supported only on 3-D devices):

            0 - Flat 2-D circular dot.

            1 - Flat-sided 3-D polyhedron.

            n - 3-D sphere with n (>1) polygon divisions per 90° of radius.

        tlen
            Maximum length of triads for gradient triad display (KEY = 3).
            Value is expressed as a fraction of the screen width (defaults to
            0.067).

        Notes
        -----
        Use /CTYPE,STAT to display the current settings.     Only the standard
        contour display [/CTYPE,0) and the isosurface contour display
        [/CTYPE,1] are supported by PowerGraphics [/GRAPHICS,POWER].

        This command is valid in any processor.
        """
        command = f"/CTYPE,{key},{dotd},{dots},{dshp},{tlen}"
        return self.run(command, **kwargs)

    def edge(self, wn="", key="", angle="", **kwargs):
        """Displays only the common lines ("edges") of an object.

        APDL Command: /EDGE

        Parameters
        ----------
        wn
            Window number (or ALL) to which command applies. The default window
            is 1.

        key
            Edge key:

            0 - Display common lines between all adjacent element faces.

            1 - Display only the common lines between non-coplanar faces (that is, show only
                the edges).

        angle
            Largest angle between two faces for which the faces are considered
            to be coplanar (0° to 180°). Defaults to 45°. A smaller angle
            produces more edges, a larger angle produces fewer edges.

        Notes
        -----
        The ANGLE field is used in PowerGraphics to determine geometric
        discontinuities. It is a tolerance measure for the differences between
        the normals of the surfaces being considered. Values within the
        tolerance are accepted as coplanar (geometrically continuous). In
        postprocessing displays, results are not averaged across discontinuous
        surfaces.

        A surface can be displayed as an edge outline without interior detail.
        This is useful for both geometry and postprocessing displays. Element
        outlines are normally shown as solid lines for geometry and
        displacement displays. Lines common to adjacent "coplanar" element
        faces are removed from the display.  Midside nodes of elements are
        ignored.

        The /SHRINK option is ignored with the edge option.

        /EDGE is not supported for PLESOL and /ESHAPE displays when in
        PowerGraphics mode (/GRAPHICS,POWER).

        The /EDGE command is valid in any processor.
        """
        command = f"/EDGE,{wn},{key},{angle}"
        return self.run(command, **kwargs)

    def eshape(
        self, scale: Union[str, int] = "", key: MapdlInt = "", **kwargs
    ) -> Optional[str]:
        """Displays elements with shapes determined from the real constants or section definition.

        APDL Command: /ESHAPE


        Parameters
        ----------
        scale
            Scaling factor:

            * 0 - Use simple display of line and area elements. This
              value is the default.

            * 1 - Use real constants or section definition to form a
                solid shape display of the applicable elements.

            FAC - Multiply certain real constants, such as thickness,
                  by FAC (where FAC > 0.01) and use them to form a
                  solid shape display of elements.

        key
            Current shell thickness key:

            * 0 - Use current thickness in the displaced solid shape
                  display of shell elements (valid for SHELL181,
                  SHELL208, SHELL209, and SHELL281). This value is the
                  default.

            * 1 - Use initial thickness in the displaced solid shape
              display of shell elements.

        Notes
        -----
        The /ESHAPE command allows beams, shells, current sources, and
        certain special-purpose elements to be displayed as solids
        with the shape determined from the real constants or section
        types. Elements are displayed via the EPLOT command. No checks
        for valid or complete input are made for the display.

        Following are details about using this command with various
        element types:

        SOLID65 elements are displayed with internal lines that
        represent rebar sizes and orientations (requires vector mode
        [/DEVICE] with a basic type of display [/TYPE,,BASIC]). The
        rebar with the largest volume ratio in each element plots as a
        red line, the next largest as green, and the smallest as blue.

        COMBIN14, COMBIN39, and MASS21 are displayed with a graphics
        icon, with the offset determined by the real constants and
        KEYOPT settings.

        BEAM188, BEAM189, PIPE288, PIPE289 and ELBOW290 are displayed
        as solids with the shape determined via the section-definition
        commands (SECTYPE and SECDATA). The arbitrary section option
        (Subtype = ASEC) has no definite shape and appears as a thin
        rectangle to show orientation. The elements are displayed with
        internal lines representing the cross- section mesh.

        SOLID272 and SOLID273 are displayed as solids with the shape
        determined via the section-definition commands (SECTYPE and
        SECDATA).  The 2-D master plane is revolved around the
        prescribed axis of symmetry.

        Contour plots are available for these elements in
        postprocessing for PowerGraphics only (/GRAPHICS,POWER). To
        view 3-D deformed shapes for the elements, issue OUTRES,MISC
        or OUTRES,ALL for static or transient analyses. To view 3-D
        mode shapes for a modal or eigenvalue buckling analysis,
        expand the modes with element results calculation ON (Elcalc =
        YES for MXPAND).

        SOURC36, CIRCU124, and TRANS126 elements always plot using
        /ESHAPE when PowerGraphics is activated (/GRAPHICS,POWER).

        In most cases, /ESHAPE renders a thickness representation of
        your shell, plane and layered elements more readily in
        PowerGraphics (/GRAPHICS,POWER). This type of representation
        employs PowerGraphics to generate the enhanced representation,
        and will often provide no enhancement in Full Graphics
        (/GRAPHICS,FULL). This is especially true for POST1 results
        displays, where /ESHAPE is not supported for most element
        types with FULL graphics.

        When PowerGraphics is active, /ESHAPE may degrade the image if
        adjacent elements have overlapping material, such as shell
        elements which are not co-planar. Additionally, if adjacent
        elements have different thicknesses, the polygons depicting
        the connectivity between the "thicker" and "thinner" elements
        along the shared element edges may not always be displayed.

        For POST1 results displays (such as PLNSOL), the following
        limitations apply:

        Rotational displacements for beam elements are used to create
        a more realistic displacement display. When /ESHAPE is active,
        displacement plots (via PLNSOL,U,X and PLDISP, for example)
        may disagree with your PRNSOL listings. This discrepancy will
        become more noticeable when the SCALE value is not equal to
        one.

        When shell elements are not co-planar, the resulting PLNSOL
        display with /ESHAPE will actually be a PLESOL display as the
        non-coincident pseudo-nodes are not averaged. Additionally,
        /ESHAPE should not be used with coincident elements because
        the plot may incorrectly average the displacements of the
        coincident elements.

        When nodes are initially coincident and PowerGraphics is
        active, duplicate polygons are eliminated to conserve display
        time and disk space. The command may degrade the image if
        initially coincident nodes have different displacements. The
        tolerance for determining coincidence is 1E-9 times the
        model’s bounding box diagonal.

        If you want to view solution results (PLNSOL, etc.) on layered
        elements (such as SHELL181, SOLSH190, SOLID185 Layered Solid,
        SOLID186 Layered Solid, SHELL208, SHELL209, SHELL281, and
        ELBOW290), set KEYOPT(8) = 1 for the layer elements so that
        the data for all layers is stored in the results file.

        You can plot the through-thickness temperatures of elements
        SHELL131 and SHELL132 regardless of the thermal DOFs in use by
        issuing the PLNSOL,TEMP command (with PowerGraphics and
        /ESHAPE active).

        The /ESHAPE,1 and /ESHAPE,FAC commands are incompatible with
        the /CYCEXPAND command used in cyclic symmetry analyses.

        This command is valid in any processor.

        """
        warnings.warn(
            "pymapdl does not support /ESHAPE when plotting in "
            "Python using ``mapdl.eplot()``.  "
            "Use ``mapdl.eplot(vtk=False)`` "
        )
        command = f"/ESHAPE,{scale},{key}"
        return self.run(command, **kwargs)

    def facet(self, lab="", **kwargs):
        """Specifies the facet representation used to form solid model displays.

        APDL Command: /FACET

        Parameters
        ----------
        lab
            Valid labels:

            FINE - Use finer tessellation to increase the number of facets for the display.
                   Provides the best representation (but decreases speed of
                   operation).

            NORML - Use the basic number of facets for the display (default).

            COAR - Use a limited number of facets for the display. This option will increase the
                   speed of the operations, but may produce poor
                   representations for some imported models.

            WIRE - Display model with a wireframe representation (fast, but surfaces will not be
                   shown).

        Notes
        -----
        Specifies the facet (or polygon) representation used to form solid
        model displays.  Used only with the APLOT, ASUM, VPLOT, and VSUM
        commands.

        This command is valid in any processor.
        """
        command = f"/FACET,{lab}"
        return self.run(command, **kwargs)

    def gline(self, wn="", style="", **kwargs):
        """Specifies the element outline style.

        APDL Command: /GLINE

        Parameters
        ----------
        wn
            Window number (or ALL) to which command applies (defaults to 1).

        style
            Outline key:

             0  - Solid element outlines (default)

             1  - Dashed element outlines

            -1  - No element outlines

        Notes
        -----
        Determines the element outline style.  Often used when node numbers are
        displayed to prevent element lines from overwriting node numbers.

        Unless you are using an OpenGL or Starbase driver, the dashed element
        outline option (/GLINE,WN,1) is not available in the following
        situations:

        Z-buffered displays (/TYPE,WN,6).

        Capped Z-buffered displays (/TYPE,WN,7).

        Qslice Z-buffered displays (/TYPE,WN,8).

        This command is valid in any processor.
        """
        command = f"/GLINE,{wn},{style}"
        return self.run(command, **kwargs)

    def gmarker(self, curve="", key="", incr="", **kwargs):
        """Specifies the curve marking style.

        APDL Command: /GMARKER

        Parameters
        ----------
        curve
            Curve number markers will be applied on (integer value between 1
            and 10).

        key
            Marker key:

            0 - No markers will be applied (default).

            1 - TRIANGLES will be applied.

            2 - SQUARES will be applied.

            3 - DIAMONDS  will be applied.

            4 - CROSSES will be applied.

        incr
            Determines the curve marking frequency. (a whole number value
            between 1 and 255). If INCR = 1, markers are displayed at every
            data point on the curve. If INCR = 2 then markers are displayed at
            every second data point. If INCR = 3 then they are displayed at
            every third data point.

        Notes
        -----
        The user-specified markers will not be drawn when the area under the
        curve is color-filled (/GROPT, FILL).
        """
        command = f"/GMARKER,{curve},{key},{incr}"
        return self.run(command, **kwargs)

    def gmface(self, lab="", n="", **kwargs):
        """Specifies the facet representation used to form solid models.

        APDL Command: GMFACE

        Parameters
        ----------
        lab
            Valid Labels:

            FINE - Value that determines how coarse the facets will be.

        n
            An integer value between one (small) and ten (large) that
            determines the tolerances that will be applied to the creation of
            arcs and surfaces. Ten will create many facets, which may in turn
            cause ANSYS to run very slowly. One will create fewer facets, which
            may in turn cause larger tolerance errors.
        """
        command = f"GMFACE,{lab},{n}"
        return self.run(command, **kwargs)

    def light(self, wn="", num="", int_="", xv="", yv="", zv="", refl="", **kwargs):
        """Specifies the light direction for the display window.

        APDL Command: /LIGHT

        Parameters
        ----------
        wn
            Window number (or ALL) to which command applies (defaults to 1).

        num
            Ambient or directional light key:

            0 - Ambient light (default).

            1 - Directional light.

        int\\_
            Light intensity factor (defaults to 0.3 for ambient, 1.0 for
            directional). This option is valid only for 3-D devices).

        xv, yv, zv
            Light direction (valid only for NUM = 1).  The directional light
            source is parallel to the line from point XV, YV, ZV to the origin,
            in the global Cartesian system origin.  Defaults to the viewing
            direction [/VIEW].

        refl
            Light reflectance factor (valid only for NUM = 1 and 3-D devices).

        Notes
        -----
        Defines the light direction for the window.  Use this command only with
        3-D graphics devices or 2-D devices when Z-buffering is used [/TYPE,,(6
        or 7)].  The ambient light has no direction, only an intensity.  You
        can position the directional light source  by defining a point (in the
        global Cartesian coordinate system) representing a point along the
        light directional line.  This point, and the global Cartesian
        coordinate system origin, define the line along which the light is
        positioned looking toward the origin.  You can use any point along the
        light line; for example, both (1.,1.,1.) and (2.,2.,2.) give the same
        light effect.  For 3-D graphics devices only, the directional light
        source also has intensity and reflectance factors.

        By choosing the highest intensity ambient light for 3-D graphics
        devices (via the command /LIGHT,WN,0,1), you can nullify color shading
        and other effects of directional lighting.

        This command is valid in any processor.
        """
        command = f"/LIGHT,{wn},{num},{int_},{xv},{yv},{zv},{refl}"
        return self.run(command, **kwargs)

    def normal(self, wn="", key="", **kwargs):
        """Allows displaying area elements by top or bottom faces.

        APDL Command: /NORMAL

        Parameters
        ----------
        wn
            Window number (or ALL) to which command applies (defaults to 1).

        key
            Display key:

            0 - No face distinction.

            1 - Show only area elements having their positive normals directed toward the
                viewing point.

            -1 - Show only area elements having their positive normals directed away from the
                 viewing point.

        Notes
        -----
        /NORMAL allows you to select area elements and area plots by the top or
        bottom faces.  It is useful for checking the normal directions on shell
        elements.  The positive normal (element Z direction) is defined by the
        right-hand rule following the node I, J, K, L input direction.  This
        command is available only with raster or hidden-line displays, for
        WIN32 or X11 2-D displays only.

        This command is valid in any processor.
        """
        command = f"/NORMAL,{wn},{key}"
        return self.run(command, **kwargs)

    def shade(self, wn="", type_="", **kwargs):
        """Defines the type of surface shading used with Z-buffering.

        APDL Command: /SHADE

        Parameters
        ----------
        wn
            Window number (or ALL) to which command applies (defaults to 1).

        type\\_
            Shading type:

            FACET or 0 - Facet shading (one color per area face) (default).

            GOURAUD or 1 - Gouraud smooth shading (smooth variation of color based on interpolated vertex
                           colors).

            PHONG or 2 - Phong smooth shading (smooth variation of color based on interpolated vertex
                         normals).

        Notes
        -----
        Defines the type of surface shading used on area, volume, and
        PowerGraphics [/GRAPHICS,POWER] displays when software Z-buffering is
        enabled [/TYPE].  This command is only functional for 2-D display
        devices.

        This command is valid in any processor.
        """
        command = f"/SHADE,{wn},{type_}"
        return self.run(command, **kwargs)

    def slashtype(self, wn="", type_="", **kwargs):
        """Defines the type of display.

        APDL Command: /TYPE

        Parameters
        ----------
        wn
            Window number (or ALL) to which command applies (defaults to 1).

        type_
            Display type. Defaults to `ZBUF` for raster mode displays or `BASIC`
            for vector mode displays:

            BASIC or 0
                Basic display (no hidden or section operations).

            SECT or 1
                Section display (plane view). Use the `/CPLANE` command to
                define the cutting plane.

            HIDC or 2
                Centroid hidden display (based on item centroid sort).

            HIDD or 3
                Face hidden display (based on face centroid sort).

            HIDP or 4
                Precise hidden display (like `HIDD` but with more precise checking).
                Because all facets are sorted, this mode can be extremely slow,
                especially for large models.

            CAP or 5
                Capped hidden display (same as combined `SECT` and `HIDD` with
                model in front of section plane removed).

            ZBUF or 6
                Z-buffered display (like `HIDD` but using software Z-buffering).

            ZCAP or 7
                Capped Z-buffered display (same as combined `SECT` and `ZBUF`
                with model in front of section plane removed).

            ZQSL or 8
                `QSLICE` Z-buffered display (same as `SECT` but the edge lines
                of the remaining 3D model are shown).

            HQSL or 9
                `QSLICE` precise hidden display (like `ZQSL` but using precise hidden).

        Notes
        -----
        **Command Default:** `ZBUF` for raster mode displays; `BASIC` for vector
        mode displays.

        Defines the type of display, such as section display or hidden-line
        display. Use the `/DEVICE` command to specify either raster or
        vector mode.

        The `SECT`, `CAP`, `ZCAP`, `ZQSL`, and `HQSL` options produce section
        displays. The section or "cutting" plane is specified on the `/CPLANE `
        command as either normal to the viewing vector at the focus point
        (default), or as the working plane.

        When you use PowerGraphics, the section display options (`Section`,
        `Slice`, and `Capped`) use different averaging techniques for the
        interior and exterior results. Because of the different averaging
        schemes, anomalies may appear at the transition areas. In many cases,
        the automatically computed `MIN` and `MAX` values will differ from the
        full range of interior values. You can lessen the effect of these
        anomalies by issuing` AVRES,,FULL` (Main Menu> General Post Proc>
        Options for Outp). This command sets your legend's automatic contour
        interval range according to the minimum and maximum results found
        throughout the entire model.

        With PowerGraphics active (`/GRAPHICS,POWER`), the averaging scheme for
        surface data with interior element data included (`AVRES,,FULL`) and
        multiple facets per edge (`/EFACET,2` or `/EFACET,4`) will yield
        differing minimum and maximum contour values depending on the Z-
        Buffering options (`/TYPE,,6` or `/TYPE,,7`). When the Section data is
        not included in the averaging schemes (`/TYPE,,7`), the resulting
        absolute value for the midside node is significantly smaller.

        The `HIDC`, `HIDD`, `HIDP`, `ZBUF`, `ZQSL`, and `HQSL` options produce
        displays with "hidden" lines removed. Hidden lines are lines obscured
        from view by another element, area, etc. The choice of non-Z-buffered
        hidden-line procedure types is available only for raster mode
        (`/DEVICE`) displays. For vector mode displays, all non-Z-buffered
        "hidden-line" options use the same procedure (which is slightly
        different from the raster procedures). Both geometry and postprocessing
        displays may be of the hidden- line type. Interior stress contour lines
        within solid elements can also be removed as hidden lines, leaving only
        the stress contour lines and element outlines on the visible surfaces.
        Midside nodes of elements are ignored on postprocessing displays.
        Overlapping elements will not be displayed.

        The `ZBUF`, `ZCAP`, and `ZQSL` options use a specific hidden-line
        technique called software Z-buffering. This technique allows a more
        accurate display of overlapping surfaces (common when using Boolean
        operations or `/ESHAPE` on element displays), and allows smooth shaded
        displays on all interactive graphics displays. Z-buffered displays can
        be performed faster than `HIDP` and `CAP` type displays for large
        models. See also the `/LIGHT`, `/SHADE`, and `/GFILE` commands for
        additional options when Z-buffering is used.

        This command is valid in any processor.
        """
        command = f"/TYPE,{wn},{type_}"
        return self.run(command, **kwargs)

    def trlcy(self, lab="", tlevel="", n1="", n2="", ninc="", **kwargs):
        """Specifies the level of translucency.

        APDL Command: /TRLCY

        Parameters
        ----------
        lab
            Apply translucency level to the items specified by the following
            labels:

            ELEM - Elements.  Use N1, N2, NINC fields for element numbers.

            AREA - Solid model areas.  Use N1, N2, NINC fields for area numbers.

            VOLU - Solid model volumes.  Use N1, N2, NINC fields for volume numbers.

            ISURF - Isosurfaces (surfaces of constant stress, etc., value).  Translucency varies
                    with result value, to a maximum of the specified
                    translucency level.

            CM - Component group.  Use N1 for component name, ignore N2 and NINC.

            CURVE - Filled areas under curves of line graphs.  Use N1, N2, NINC fields for curve
                    numbers.

            ZCAP - If /TYPE,WN,ZCAP is the current display type, then /TRLCY,ZCAP,TLEVEL will
                   display the model in window WN with the portion of the model
                   in front of the section plane displayed at the translucency
                   level TLEVEL.

            ON, OFF - Sets the specified translucency display on or off. All other fields are
                      ignored.

        tlevel
            Translucency level:  0.0 (opaque) to 1.0 (transparent).

        n1, n2, ninc
            Used only with labels as noted above.  Apply translucency level to
            Lab items numbered N1 to N2 (defaults to N1) in steps of NINC
            (defaults to 1).  If N1 is blank or ALL, apply specified
            translucency level to entire selected range.  If Lab is CM, use
            component name for N1 and ignore N2 and NINC. A value of N1 = P
            allows you to graphically pick elements, areas, and volumes. You
            can then assign translucency levels to the entities via the picker.
            The Lab and TLEVEL fields are ignored when translucency is applied
            by picking.

        Notes
        -----
        Specifies the level of translucency for various items.  Issue
        /TRLCY,DEFA to reset the default (0) translucency levels.  This command
        is valid only on selected 2-D and 3-D graphics devices; see  in the
        Basic Analysis Guide for more information on applying translucency.

        For 2-D devices, ANSYS displays only the visible faces of the items
        being displayed.  The information behind the facing planes is not
        displayed.  Issuing the /SHRINK command will force the hardware to
        display information behind the translucent items.

        This command is valid in any processor.
        """
        command = f"/TRLCY,{lab},{tlevel},{n1},{n2},{ninc}"
        return self.run(command, **kwargs)



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import click


@click.command(
    short_help="Stop MAPDL instances.",
    help="""This command stop MAPDL instances running on a given port or with a given process id (PID).

By default, it stops instances running on the port 50052.""",
)
@click.option(
    "--port",
    default=None,
    type=int,
    help="Port where the MAPDL instance is running.",
)
@click.option(
    "--pid",
    default=None,
    type=int,
    help="Process PID where the MAPDL instance is running.",
)
@click.option(
    "--all",
    is_flag=True,
    flag_value=True,
    type=bool,
    default=False,
    help="Kill all MAPDL instances",
)
def stop(port, pid, all):
    import psutil

    from ansys.mapdl.core.launcher import is_ansys_process

    PROCESS_OK_STATUS = [
        # List of all process status, comment out the ones that means that
        # process is not OK.
        # If process is OK, it means it can be killed normally.
        psutil.STATUS_RUNNING,  #
        psutil.STATUS_SLEEPING,  #
        psutil.STATUS_DISK_SLEEP,  #
        # psutil.STATUS_STOPPED, #
        # psutil.STATUS_TRACING_STOP, #
        # psutil.STATUS_ZOMBIE, #
        psutil.STATUS_DEAD,  #
        # psutil.STATUS_WAKE_KILL, #
        # psutil.STATUS_WAKING, #
        psutil.STATUS_PARKED,  # (Linux)
        psutil.STATUS_IDLE,  # (Linux, macOS, FreeBSD)
        # psutil.STATUS_LOCKED, # (FreeBSD)
        # psutil.STATUS_WAITING, # (FreeBSD)
        # psutil.STATUS_SUSPENDED, # (NetBSD)
    ]

    if not pid and not port:
        port = 50052

    if port or all:
        killed_ = False
        for proc in psutil.process_iter():
            try:
                if (
                    psutil.pid_exists(proc.pid)
                    and proc.status() in PROCESS_OK_STATUS
                    and is_ansys_process(proc)
                ):
                    # Killing "all"
                    if all:
                        try:
                            proc.kill()
                            killed_ = True
                        except psutil.NoSuchProcess:
                            pass

                    else:
                        # Killing by ports
                        if str(port) in proc.cmdline():
                            try:
                                proc.kill()
                                killed_ = True
                            except psutil.NoSuchProcess:
                                pass

            except psutil.NoSuchProcess:
                continue

        if all:
            str_ = ""
        else:
            str_ = f" running on port {port}"

        if not killed_:
            click.echo(
                click.style("ERROR: ", fg="red")
                + "No Ansys instances"
                + str_
                + " have been found."
            )
        else:
            click.echo(
                click.style("Success: ", fg="green")
                + "Ansys instances"
                + str_
                + " have been stopped."
            )
        return

    if pid:
        try:
            pid = int(pid)
        except ValueError:
            click.echo(
                click.style("ERROR: ", fg="red")
                + "PID provided could not be converted to int."
            )

        p = psutil.Process(pid)
        for child in p.children(recursive=True):
            child.kill()
        p.kill()

        if p.status == "running":
            click.echo(
                click.style("ERROR: ", fg="red")
                + f"The process with PID {pid} and its children could not be killed."
            )
        else:
            click.echo(
                click.style("Success: ", fg="green")
                + f"The process with PID {pid} and its children have been stopped."
            )
        return



# Copyright (C) 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class Annotation:
    def an3d(self, kywrd="", key="", **kwargs):
        """Specifies 3-D annotation functions

        APDL Command: /AN3D

        Parameters
        ----------
        num
            Unique number assigned as each annotation is applied to a model.
            These numbers are applied sequentially, although when an annotation
            entity is deleted, its number is reassigned.

        type
            Annotation internal type number (101 = text, 102 = line, 103 =
            point, 104 = area, 105 = arrow, 106 = symbol, 108 = bitmap).

        xhot, yhot, zhot
            X, Y, Z coordinates for hot spot location.

        Notes
        -----
        Because 3-D annotation is applied in relation to the XYZ coordinates of
        the anchor, you can transform your model, and the annotation will
        maintain the spatial relationship with the model. This works within
        reason, and there are instances where changing the perspective or the
        size of the model will change the apparent relationship between the
        annotation and the model.

        The overall 3-D dimensions of your model are defined by a bounding
        box.  If portions of your model's bounding box lie outside of the
        visible area of your graphics window (if you are zoomed in on a
        specific area of your model), it can affect the placement of your 3-D
        annotations.  Zooming out will usually overcome this problem.

        3-D annotation is valid for the Cartesian (CSYS,0) coordinate system
        only. If you want to annotate a model you created in another coordinate
        system, use 2-D annotation (note that 2-D annotations do not remain
        anchored for dynamic rotations or transformations).

        When you apply user defined bitmaps, the size of the annotation can
        vary. Use the options menu of the 3-D annotation widget to adjust the
        size and placement of your bitmaps.

        You cannot use the "!" and "$" characters in ANSYS text annotation.

        The GUI generates this command during 3-D annotation operations and
        inserts the command into the log file (Jobname.LOG). You should NOT
        type this command directly during an ANSYS session (although the
        command can be included in an input file for batch input or for use
        with the /INPUT command).
        """
        command = f"/AN3D,{kywrd},{key}"
        return self.run(command, **kwargs)

    def annot(self, lab="", val1="", val2="", **kwargs):
        """Activates graphics for annotating displays (GUI).

        APDL Command: /ANNOT

        Parameters
        ----------
        lab
            Annotation control key:

            OFF
                Turns off annotation for each subsequent display (default).

            ON
                Turns on annotation for each subsequent display.

            DELE
                Deletes all annotation.

            SAVE
                Saves annotation on a file.  Use VAL1 for file name (defaults to Jobname) and
                VAL2 for the extension (defaults to ANO).

            SCALE
                Sets annotation scale factor (direct input only).  Use VAL1 for value (0.1 to
                10.0) (defaults to 1.0).

            XORIG
                Sets the annotation x origin (direct input only).  Use VAL1 for value (-3.0 to
                3.0).

            YORIG
                Sets annotation y origin (direct input only).  Use VAL1 for value (-3.0 to
                3.0).

            SNAP
                Sets annotation snap (menu button input only).  Use VAL1 for value (0.002 to
                0.2) (defaults to 0.002).

            STAT
                Displays current annotation status.

            DEFA
                Sets annotation specifications to the default values.

            REFR
                Redisplays annotation graphics.

            TMOD
                Sets the annotation text mode. If VAL1 = 1, annotation text will be drawn in
                scalable bitmap fonts (default). If VAL1 = 0, annotation
                text will be drawn with stroke text.

        val1
            Value (or file name) as noted with label above.

        val2
            Value (or file name extension) as noted with label above.

        Notes
        -----
        This is a command generated by the GUI and will appear in the log file
        (Jobname.LOG) if annotation is used.  This command is not intended to
        be typed in directly in an ANSYS session (although it can be included
        in an input file for batch input or for use with the /INPUT command).

        You cannot use the "!" and "$" characters in ANSYS text annotation.

        /ANNOT activates annotation graphics for adding annotation to displays.
        Commands representing the annotation instructions are automatically
        created by the annotation functions in the GUI and written to
        Jobname.LOG.  The annotation commands are /ANNOT, /ANUM, /TLABEL,
        /LINE, /LARC, /LSYMBOL, /POLYGON, /PMORE, /PCIRCLE, /PWEDGE, /TSPEC,
        /LSPEC, and /PSPEC.  Annotation graphics are relative to the full
        Graphics Window and are not affected by ANSYS window-specific commands
        (/WINDOW, /VIEW, etc.).

        This command is valid in any processor.
        """
        command = f"/ANNOT,{lab},{val1},{val2}"
        return self.run(command, **kwargs)

    def anum(self, num="", type_="", xhot="", yhot="", **kwargs):
        """Specifies the annotation number, type, and hot spot (GUI).

        APDL Command: /ANUM

        Parameters
        ----------
        num
            Annotation number.  ANSYS automatically assigns the lowest
            available number.  You cannot assign a higher number if a lower
            number is available; ANSYS will substitute the lowest available
            number in place of any user-specified higher number.

        type\\_
            Annotation internal type number.  If TYPE = DELE, delete annotation
            NUM.

            1
                Text

            2
                Block text (not available in GUI)

            3
                Dimensions

            4
                Lines

            5
                Rectangles

            6
                Circles

            7
                Polygons

            8
                Arcs

            9
                Wedges, pies

            11
                Symbols

            12
                Arrows

            13
                Bitmap

        xhot
            X hot spot (-1.0 < X < 2.0).  Used for menu button item delete.

        yhot
            Y hot spot (-1.0 < Y < 1.0).  Used for menu button item delete.

        Notes
        -----
        This is a command generated by the GUI and will appear in the log file
        (Jobname.LOG) if annotation is used.  This command is not intended to
        be typed in directly in an ANSYS session (although it can be included
        in an input file for batch input or for use with the /INPUT command).

        Type 13 (bitmap) annotation applies user defined bitmaps defined using
        the FILE option of the /TXTRE command.

        This command is valid in any processor.
        """
        command = f"/ANUM,{num},{type_},{xhot},{yhot}"
        return self.run(command, **kwargs)

    def slashlarc(self, xcentr="", ycentr="", xlrad="", angle1="", angle2="", **kwargs):
        """Creates annotation arcs (GUI).

        APDL Command: /LARC

        Parameters
        ----------
        xcentr
            Arc X center location (-1.0 < X < 1.0).

        ycentr
            Arc Y center location (-1.0 < Y < 1.0).

        xlrad
            Arc radius length.

        angle1
            Starting angle of arc.

        angle2
            Ending angle of arc. The arc is drawn counterclockwise from the
            starting angle, `angle1`, to the ending angle, `angle2`.

        Notes
        -----
        This command defines annotation arcs to be written directly onto the
        display at a specified location. The command is generated by the Graphical
        User Interface (GUI) and will appear in the log file (`Jobname.LOG`) if
        annotation is used. It is not intended to be typed in directly in a
        Mechanical APDL session (although it can be included in an input file
        for batch input or for use with the `/INPUT` command).
        All arcs are shown on subsequent displays unless the annotation is
        turned off or deleted. Issueu `/LSPEC` to set the attributes
        of the arc.

        This command is valid in any processor.
        """
        command = f"/LARC, {xcentr}, {ycentr}, {xlrad}, {angle1}, {angle2}"
        return self.run(command, **kwargs)

    def slashline(self, x1="", y1="", x2="", y2="", **kwargs):
        """Creates annotation lines (GUI).

        APDL Command: /LINE

        Parameters
        ----------
        X1
            Line X starting location (-1.0 < X < 2.0).
        Y1
            Line Y starting location (-1.0 < Y < 1.0).
        X2
            Line X ending location (-1.0 < X < 2.0).
        Y2
            Line Y ending location (-1.0 < Y < 1.0).

        Notes
        -----
        This command defines annotation lines to be written directly onto the
        display at a specified location. The command is generated by the Graphical
        User Interface (GUI) and appears in the log file (`Job-name.LOG`) if
        annotation is used. It is not intended to be typed in directly in a
        Mechanical APDL session (although it can be included in an input file
        for batch input or for use with the `/INPUT` command). All lines are
        shown on subsequent displays unless the annotation is turned off or
        deleted. Issue `/LSPEC` to set the attributes of the line.

        This command is valid in any processor.
        """
        command = f"/LINE, {x1}, {y1}, {x2}, {y2}"
        return self.run(command, **kwargs)

    def lspec(self, lcolor="", linstl="", xlnwid="", **kwargs):
        """Specifies annotation line attributes (GUI).

        APDL Command: /LSPEC

        Parameters
        ----------
        lcolor
            Line color:

            0
                Black

            1
                Red-Magenta

            2
                Magenta

            3
                Blue-Magenta

            4
                Blue

            5
                Cyan-Blue

            6
                Cyan

            7
                Green-Cyan

            8
                Green

            9
                Yellow-Green

            10
                Yellow

            11
                Orange

            12
                Red

            13
                Dark Gray

            14
                Light Gray

            15
                White

        linstl
            Line style:

            0
                Solid line.

            1
                Dashed line.

        xlnwid
            Line width multiplier (1.0 to 20.0).  Defaults to 1.0.

        Notes
        -----
        Specifies annotation line attributes to control certain characteristics
        of the lines created via the /LINE, /LARC, /LSYMBOL, /POLYGON, /PMORE,
        /PCIRCLE, and /PWEDGE commands.  This is a command generated by the
        Graphical User Interface (GUI) and will appear in the log file
        (Jobname.LOG) if annotation is used.  This command is not intended to
        be typed in directly in an ANSYS session (although it can be included
        in an input file for batch input or for use with the /INPUT command).

        This command is valid in any processor.
        """
        command = f"/LSPEC,{lcolor},{linstl},{xlnwid}"
        return self.run(command, **kwargs)

    def lsymbol(self, x="", y="", symang="", symtyp="", symsiz="", keybmp="", **kwargs):
        """Creates annotation symbols (GUI).

        APDL Command: /LSYMBOL

        Parameters
        ----------
        x
            X location for symbol (-1.0 < X < 2.0).

        y
            Y location for symbol (-1.0 < Y < 1.0).

        symang
            Symbol orientation angle.

        symtyp
            Symbol type:

            1
                Arrow.

            2
                Tee.

            3
                Circle.

            4
                Triangle.

            5
                Star.

        symsiz
            Symbol size multiplier (0.1 to 20.0).  Defaults to 1.0.

        keybmp
            If KEYBMP = 1, the annotation is a bitmap. SYMTYP will then be a
            number from 1-99, indicating the bitmap type (see notes), and X and
            Y will define the lower left corner of the bitmap. The SYMANG,
            SYMSIZarguments are ignored. If KEYBMP = 0, or blank, then the
            argument definitions above apply.

        Notes
        -----
        Defines annotation symbols to be written directly onto the display at a
        specified location.  This is a command generated by the GUI and will
        appear in the log file (Jobname.LOG) if annotation is used.  This
        command is not intended to be typed in directly in an ANSYS session
        (although it can be included in an input file for batch input or for
        use with the /INPUT command).

        All symbols are shown on subsequent displays unless the annotation is
        turned off or deleted.  Use the /LSPEC command to set the attributes of
        the symbol.

        The KEYBMP argument reads the symtype argument to determine which
        bitmap to insert. This bitmap is defined by an integer between 1 and
        99. Numbers 1 through 40 correspond to the standard texture values
        found in the /TXTRE  command, while numbers 51 through 99 correspond to
        user supplied bitmaps, as defined using the Filename option of the
        /TXTRE command. Numbers 51 through 57 are predefined (the logos, clamps
        and arrows available from the GUI) but can be overridden. Numbers 41
        through 50 are reserved.

        This command is valid in any processor.
        """
        command = f"/LSYMBOL,{x},{y},{symang},{symtyp},{symsiz},{keybmp}"
        return self.run(command, **kwargs)

    def slash_pcircle(self, xcentr="", ycentr="", xlrad="", **kwargs):
        """Creates an annotation circle (GUI).

        APDL Command: /PCIRCLE

        Parameters
        ----------
        xcentr
            Circle X center location (-1.0 < X < 2.0).

        ycentr
            Circle Y center location (-1.0 < Y < 1.0).

        xlrad
            Circle radius length.

        Notes
        -----
        Creates an annotation circle to be written directly onto the display at
        a specified location.  This is a command generated by the Graphical
        User Interface (GUI) and will appear in the log file (Jobname.LOG) if
        annotation is used.  This command is not intended to be typed in
        directly in an ANSYS session (although it can be included in an input
        file for batch input or for use with the /INPUT command).

        All circles are shown on subsequent displays unless the annotation is
        turned off or deleted.  Use the /LSPEC and the /PSPEC command to set
        the attributes of the circle.

        This command is valid in any processor.
        """
        command = f"/PCIRCLE,{xcentr},{ycentr},{xlrad}"
        return self.run(command, **kwargs)

    def pmore(self, x5="", y5="", x6="", y6="", x7="", y7="", x8="", y8="", **kwargs):
        """Creates an annotation polygon (GUI).

        APDL Command: /PMORE

        Parameters
        ----------
        x5
            X location for vertex 5 of polygon (-1.0 < X < 2.0).

        y5
            Y location for vertex 5 of polygon (-1.0 < Y < 1.0).

        x6
            X location for vertex 6 of polygon (-1.0 < X < 2.0).

        y6
            Y location for vertex 6 of polygon (-1.0 < Y < 1.0).

        x7
            X location for vertex 7 of polygon (-1.0 < X < 2.0).

        y7
            Y location for vertex 7 of polygon (-1.0 < Y < 1.0).

        x8
            X location for vertex 8 of polygon (-1.0 < X < 2.0).

        y8
            Y location for vertex 8 of polygon (-1.0 < Y < 1.0).

        Notes
        -----
        Defines the 5th through 8th vertices of an annotation polygon
        [/POLYGON].  This is a command generated by the Graphical User
        Interface (GUI) and will appear in the log file (Jobname.LOG) if
        annotation is used.  This command is not intended to be typed in
        directly in an ANSYS session (although it can be included in an input
        file for batch input or for use with the /INPUT command).

        This command is valid in any processor.
        """
        command = f"/PMORE,,{x5},{y5},{x6},{y6},{x7},{y7},{x8},{y8}"
        return self.run(command, **kwargs)

    def polygon(
        self,
        nvert="",
        x1="",
        y1="",
        x2="",
        y2="",
        x3="",
        y3="",
        x4="",
        y4="",
        **kwargs,
    ):
        """Creates annotation polygons (GUI).

        APDL Command: /POLYGON

        Parameters
        ----------
        nvert
            Number of vertices of polygon (3  NVERT   8).  Use /PMORE for
            polygons with more than 4 vertices.

        x1
            X location for vertex 1 of polygon (-1.0 < X < 2.0).

        y1
            Y location for vertex 1 of polygon (-1.0 < Y < 1.0).

        x2
            X location for vertex 2 of polygon (-1.0 < X < 2.0).

        y2
            Y location for vertex 2 of polygon (-1.0 < Y < 1.0).

        x3
            X location for vertex 3 of polygon (-1.0 < X < 2.0).

        y3
            Y location for vertex 3 of polygon (-1.0 < Y < 1.0).

        x4
            X location for vertex 4 of polygon (-1.0 < X < 2.0).

        y4
            Y location for vertex 4 of polygon (-1.0 < Y < 1.0).

        Notes
        -----
        Creates annotation polygons to be written directly onto the display at
        a specified location.  This is a command generated by the Graphical
        User Interface (GUI) and will appear in the log file (Jobname.LOG) if
        annotation is used.  This command is not intended to be typed in
        directly in an ANSYS session (although it can be included in an input
        file for batch input or for use with the /INPUT command).

        All polygons are shown on subsequent displays unless the annotation is
        turned off or deleted.  Use the /LSPEC and the /PSPEC command to set
        the attributes of the polygon.  Use the /PMORE command to define the
        5th through 8th vertices of the polygon.

        This command is valid in any processor.
        """
        command = f"/POLYGON,{nvert},{x1},{y1},{x2},{y2},{x3},{y3},{x4},{y4}"
        return self.run(command, **kwargs)

    def pspec(self, pcolor="", kfill="", kbordr="", **kwargs):
        """Creates annotation polygon attributes (GUI).

        APDL Command: /PSPEC

        Parameters
        ----------
        pcolor
            Polygon color (0  PCOLOR   15):

            0
                Black.

            1
                Red-Magenta.

            2
                Magenta.

            3
                Blue-Magenta.

            4
                Blue.

            5
                Cyan-Blue.

            6
                Cyan.

            7
                Green-Cyan.

            8
                Green.

            9
                Yellow-Green.

            10
                Yellow.

            11
                Orange.

            12
                Red.

            13
                Dark Gray.

            14
                Light Gray.

            15
                White.

        kfill
            Polygon fill key:

            0
                Hollow polygon.

            1
                Filled polygon.

        kbordr
            Polygon border key:

            0
                No border.

            1
                Border.

        Notes
        -----
        Creates annotation polygon attributes to control certain
        characteristics of the polygons created via the /POLYGON, /PMORE,
        /PCIRCLE and /PWEDGE commands.  This is a command generated by the
        Graphical User Interface (GUI) and will appear in the log file
        (Jobname.LOG) if annotation is used.  This command is not intended to
        be typed in directly in an ANSYS session (although it can be included
        in an input file for batch input or for use with the /INPUT command).

        This command is valid in any processor.
        """
        command = f"/PSPEC,{pcolor},{kfill},{kbordr}"
        return self.run(command, **kwargs)

    def pwedge(self, xcentr="", ycentr="", xlrad="", angle1="", angle2="", **kwargs):
        """Creates an annotation wedge (GUI).

        APDL Command: /PWEDGE

        Parameters
        ----------
        xcentr
            Wedge X center location (-1.0 < X < 2.0).

        ycentr
            Wedge Y center location (-1.0 < Y < 1.0).

        xlrad
            Wedge radius length.

        angle1
            Starting angle of wedge.

        angle2
            Ending angle of wedge.  The wedge is drawn counterclockwise from
            the starting angle, ANGLE1, to the ending angle, ANGLE2.

        Notes
        -----
        Creates an annotation wedge to be written directly onto the display at
        a specified location.  This is a command generated by the Graphical
        User Interface (GUI) and will appear in the log file (Jobname.LOG) if
        annotation is used.  This command is not intended to be typed in
        directly in an ANSYS session (although it can be included in an input
        file for batch input or for use with the /INPUT command).

        All wedges are shown on subsequent displays unless the annotation is
        turned off or deleted.  Use the /LSPEC and the /PSPEC command to set
        the attributes of the wedge.

        This command is valid in any processor.
        """
        command = f"/PWEDGE,{xcentr},{ycentr},{xlrad},{angle1},{angle2}"
        return self.run(command, **kwargs)

    def tlabel(self, xloc="", yloc="", text="", **kwargs):
        """Creates annotation text (GUI).

        APDL Command: /TLABEL

        Parameters
        ----------
        xloc
            Text X starting location (-1.0 < X < 1.6).

        yloc
            Text Y starting location (-1.0 < Y < 1.0).

        text
            Text string (60 characters maximum).  Parameter substitution may be
            forced within the text by enclosing the parameter name or
            parametric expression within percent (%) signs.

        Notes
        -----
        Defines annotation text to be written directly onto the display at a
        specified location.  This is a command generated by the Graphical User
        Interface (GUI) and will appear in the log file (Jobname.LOG) if
        annotation is used.  This command is not intended to be typed in
        directly in an ANSYS session (although it can be included in an input
        file for batch input or for use with the /INPUT command).

        All text is shown on subsequent displays unless the annotation is
        turned off or deleted.  Use the /TSPEC command to set the attributes of
        the text.

        This command is valid in any processor.
        """
        command = f"/TLABEL,{xloc},{yloc},{text}"
        return self.run(command, **kwargs)

    def tspec(self, tcolor="", tsize="", txthic="", pangle="", iangle="", **kwargs):
        """Creates annotation text attributes (GUI).

        APDL Command: /TSPEC

        Parameters
        ----------
        tcolor
            Text color (0  TCOLOR   15):

            0
                Black.

            1
                Red-Magenta.

            2
                Magenta.

            3
                Blue-Magenta.

            4
                Blue.

            5
                Cyan-Blue.

            6
                Cyan.

            7
                Green-Cyan.

            8
                Green.

            9
                Yellow-Green.

            10
                Yellow.

            11
                Orange.

            12
                Red.

            13
                Dark Gray.

            14
                Light Gray.

            15
                White.

        tsize
            Text size factor.

        txthic
            Text thickness key:

            1
                normal.

            2
                twice as thick.

            3
                three times as thick.

            4
                four times as thick.

        pangle
            Text path angle (0.0 < angle < 360.0).

        iangle
            Text italic angle (0.0 < angle < 45.0).

        Notes
        -----
        Defines annotation text attributes to control certain characteristics
        of the text created via the /TLABEL command.  This is a command
        generated by the Graphical User Interface (GUI) and will appear in the
        log file (Jobname.LOG) if annotation is used.  This command is not
        intended to be typed in directly in an ANSYS session (although it can
        be included in an input file for batch input or for use with the /INPUT
        command).

        This command is valid in any processor.
        """
        command = f"/TSPEC,{tcolor},{tsize},{txthic},{pangle},{iangle}"
        return self.run(command, **kwargs)
